/////////////////////////////////////////////////////////////////////
//
//            X   X           X
//           XXX XX         XX
//          XXXXXXXX      XXX
//         XX X XXXXXXXXXXX
//        XXXXX XXXXXXXXX
//       XXXXX XXXXXXXXXX
//            XXX XXX XXX
//           XXX XX   XX
//           X   X     X
//
//    Copyright (C) 2003-2016  Ron Jakl
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
/////////////////////////////////////////////////////////////////////

#include "graphwidget.h"

#include <QWidget>
#include <QLabel>
#include <QToolTip>
#include <QWhatsThis>
#include <QFont>
#include <QFontMetrics>
#include <QPen>
#include <QColor>
#include <QVector>
#include <QPaintEvent>
#include <QPainter>

#include <math.h>
#include <assert.h>

#include "graphwidget.h"

TGraphWidget::TGraphWidget(
	const QWidget						*Parent,
	Qt::WindowFlags 					Flags)
: QWidget(const_cast<QWidget*>(Parent),Flags)
{
	QFontMetrics						FontMetrics(font());
	
	d_GraphType = TGraphWidget::LINE_GRAPH;

	d_VLegendTitle = "V Title";
	d_HLegendTitle = "H Title";
	d_Title = "Sample Graph";

	d_VFormat = "%3.4lf";
	d_HFormat = "%3.4lf";

	d_ShowMouseDownBox = false;
	d_LeftMouseDown = false;
	d_LeftMouseDownMoving = false;
	d_RightMouseDown = false;
	d_RightMouseDownMoving = false;

	d_LineHeight = FontMetrics.height();
	
	d_CoordDisplay.sprintf("%.5lf,   %.5lf",double(0),double(0));

	d_draw_border = true;
	d_ScaleToFitOnPaint = false;
	d_ZoomOutOnPaint = false;
	
	this->setMouseTracking(true);
	
	this->setSizePolicy(QSizePolicy::MinimumExpanding,QSizePolicy::MinimumExpanding);

	d_HLegendMode = TGraphWidget::DEFAULT_LEGEND_PRECISION;
	d_VLegendMode = TGraphWidget::DEFAULT_LEGEND_PRECISION;
	
	d_y_minmax = 0.000005;
	d_lock_aspect = false;
}

TGraphWidget::~TGraphWidget()
{
}

int TGraphWidget::DataSetSize(
	const int							Set) const
{
	assert(Set < static_cast<int>(d_DataSets.size()));

	return static_cast<int>(d_DataSets[Set].Points.size());
}

TVector2 TGraphWidget::GetPoint(
	const int							Set,
	const int							Offset) const
{
	assert(Set < static_cast<int>(d_DataSets.size()));
	assert(Offset < static_cast<int>(d_DataSets[Set].Points.size()));

	return d_DataSets[Set].Points[Offset];
}

bool TGraphWidget::DeleteDataSet(
	const int 							Offset)
{
	assert(Offset < static_cast<int>(d_DataSets.size()));

	d_DataSets.erase(d_DataSets.begin() + Offset);

	return true;
}

void TGraphWidget::DeleteDataSets(void)
{
	d_DataSets.clear();
}

int TGraphWidget::CreateDataSet(
	const QColor 						Color,
	const TGraphWidget::TGraphType		dataset_type)
{
	TDataSet								Set;

	Set.Color = Color;
	Set.DatasetGraphType = dataset_type;

	d_DataSets.push_back(Set);

	return static_cast<int>(d_DataSets.size() - 1);
}

bool TGraphWidget::AddPoint(
	const int 							Set,
	const TVector2						&Point)
{
	assert(Set < static_cast<int>(d_DataSets.size()));

	d_DataSets[Set].Points.push_back(Point);

	return true;
}

bool TGraphWidget::ModifyPoint(
	const int 							Set,
	const int 							Offset,
	const TVector2						&Point)
{
	assert(Set < static_cast<int>(d_DataSets.size()));
	assert(Offset < static_cast<int>(d_DataSets[Set].Points.size()));

	d_DataSets[Set].Points[Offset] = Point;

	return true;
}

bool TGraphWidget::DeletePoint(
	const int 							Set,
	const int 							Offset)
{
	assert(Set < static_cast<int>(d_DataSets.size()));
	assert(Offset < static_cast<int>(d_DataSets[Set].Points.size()));
	
	d_DataSets[Set].Points.erase(d_DataSets[Set].Points.begin() + Offset);
	
	return true;
}

void TGraphWidget::UpdateGraph(void)
{
	if(d_Viewport.height() < 1 || d_Viewport.width() < 1)
	{
		return;
	}

	this->DrawPixmap();
	this->update();
}

bool TGraphWidget::PaintToDevice(
	QPainter 							*Device,
	const QRect							&DeviceRect,
	const bool							draw_border)
{
	TViewportMat						OrgViewportMat;
	QRect								OrgGraphRect;
	QRect								OrgVLegendRect;
	QRect								OrgVLegendTitleRect;
	QRect								OrgHLegendRect;
	QRect								OrgHLegendTitleRect;
	QRect								OrgTitleRect;
	QRect								OrgViewport;
	int									OrgLineHeight;
	bool								OrgDrawBorder;
	int									X,Y;
	
	assert(Device);
	QFontMetrics						FontMetrics(Device->font());
	
	
	OrgViewportMat = d_ViewportMat;
	OrgGraphRect = d_GraphRect;
	OrgVLegendRect = d_VLegendRect;
	OrgVLegendTitleRect = d_VLegendRect;
	OrgHLegendRect = d_HLegendRect;
	OrgHLegendTitleRect = d_HLegendTitleRect;
	OrgTitleRect = d_TitleRect;
	OrgViewport = d_Viewport;
	OrgLineHeight = d_LineHeight;
	OrgDrawBorder = d_draw_border;
	
	d_draw_border = draw_border;

	d_LineHeight = FontMetrics.height();

	d_Viewport = DeviceRect;
	
	X = d_Viewport.width();
	Y = d_Viewport.height();

	d_VLegendTitleRect.setLeft(d_Viewport.left());
	d_VLegendTitleRect.setTop(d_Viewport.top());
	d_VLegendTitleRect.setRight(d_LineHeight * 2 + d_Viewport.left());
	d_VLegendTitleRect.setBottom(Y - d_LineHeight * 2);

	d_HLegendTitleRect.setLeft(d_LineHeight * 2 + d_Viewport.left());
	d_HLegendTitleRect.setTop(Y - d_LineHeight * 2);
	d_HLegendTitleRect.setRight(X);
	d_HLegendTitleRect.setBottom(Y);

	d_TitleRect.setLeft(d_VLegendTitleRect.right() + 5 * d_LineHeight);
	d_TitleRect.setTop(d_Viewport.top());
	d_TitleRect.setRight(X);
	d_TitleRect.setBottom(d_LineHeight * 2 + d_Viewport.top());

	d_VLegendRect.setLeft(d_VLegendTitleRect.right());
	d_VLegendRect.setTop(d_Viewport.top());
	d_VLegendRect.setRight(d_VLegendTitleRect.right() + 4 * d_LineHeight);
	d_VLegendRect.setBottom(d_HLegendTitleRect.top());

	d_HLegendRect.setLeft(d_VLegendRect.right());
	d_HLegendRect.setTop(d_HLegendTitleRect.top() - d_LineHeight * 2);
	d_HLegendRect.setRight(X);
	d_HLegendRect.setBottom(d_HLegendTitleRect.top());

	d_GraphRect.setLeft(d_VLegendRect.right());
	d_GraphRect.setTop(d_TitleRect.bottom());
	d_GraphRect.setRight(X - d_LineHeight);
	d_GraphRect.setBottom(d_HLegendRect.top());

	d_VLegendTitleRect.setTop(d_GraphRect.top());
	d_VLegendTitleRect.setBottom(d_GraphRect.bottom());

	d_HLegendTitleRect.setLeft(d_GraphRect.left());
	d_HLegendTitleRect.setRight(d_GraphRect.right());
	
	this->DrawPixmap(Device);

	d_ViewportMat = OrgViewportMat;
	d_GraphRect = OrgGraphRect;
	d_VLegendRect = OrgVLegendRect; 
	d_VLegendRect = OrgVLegendTitleRect; 
	d_HLegendRect = OrgHLegendRect;
	d_HLegendTitleRect = OrgHLegendTitleRect;
	d_TitleRect = OrgTitleRect;
	d_Viewport= OrgViewport;
	d_LineHeight = OrgLineHeight;
	d_draw_border = OrgDrawBorder;
	
	return true;
}

bool TGraphWidget::PaintToDevice_GraphOnly(
	QPainter 							*Device,
	const QRect							&DeviceRect,
	const bool							draw_border)
{
	TViewportMat						OrgViewportMat;
	QRect								OrgGraphRect;
	QRect								OrgVLegendRect;
	QRect								OrgVLegendTitleRect;
	QRect								OrgHLegendRect;
	QRect								OrgHLegendTitleRect;
	QRect								OrgTitleRect;
	QRect								OrgViewport;
	int									OrgLineHeight;
	int									X,Y;
	QString								OrigVLegendTitle;
	QString								OrigHLegendTitle;
	QString								OrigTitle;
	bool								OrgDrawBorder;
	int									border(6);
	
	assert(Device);
	QFontMetrics						FontMetrics(Device->font());
	
	
	OrgViewportMat = d_ViewportMat;
	OrgGraphRect = d_GraphRect;
	OrgVLegendRect = d_VLegendRect;
	OrgVLegendTitleRect = d_VLegendRect;
	OrgHLegendRect = d_HLegendRect;
	OrgHLegendTitleRect = d_HLegendTitleRect;
	OrgTitleRect = d_TitleRect;
	OrgViewport = d_Viewport;
	OrgLineHeight = d_LineHeight;
	OrigVLegendTitle = d_VLegendTitle;
	OrigHLegendTitle = d_HLegendTitle;
	OrigTitle = d_Title;
	OrgDrawBorder = d_draw_border;
	
	d_VLegendTitle = QString();
	d_HLegendTitle = QString();
	d_Title = QString();
	d_draw_border = draw_border;
	
	d_LineHeight = FontMetrics.height();
	
	d_Viewport = DeviceRect;
	
	X = d_Viewport.width();
	Y = d_Viewport.height();
	
	d_VLegendTitleRect.setLeft(d_Viewport.left());
	d_VLegendTitleRect.setTop(d_Viewport.top());
	d_VLegendTitleRect.setRight(border + d_Viewport.left());
	d_VLegendTitleRect.setBottom(Y - border);
	
	d_HLegendTitleRect.setLeft(border + d_Viewport.left());
	d_HLegendTitleRect.setTop(Y - border);
	d_HLegendTitleRect.setRight(X);
	d_HLegendTitleRect.setBottom(Y);
	
	d_TitleRect.setLeft(d_VLegendTitleRect.right() + border);
	d_TitleRect.setTop(d_Viewport.top());
	d_TitleRect.setRight(X);
	d_TitleRect.setBottom(border + d_Viewport.top());
	
	d_VLegendRect.setLeft(d_VLegendTitleRect.right());
	d_VLegendRect.setTop(d_Viewport.top());
	d_VLegendRect.setRight(d_VLegendTitleRect.right() + 4 * d_LineHeight);
	d_VLegendRect.setBottom(d_HLegendTitleRect.top());
	
	d_HLegendRect.setLeft(d_VLegendRect.right());
	d_HLegendRect.setTop(d_HLegendTitleRect.top() - d_LineHeight * 2);
	d_HLegendRect.setRight(X);
	d_HLegendRect.setBottom(d_HLegendTitleRect.top());
	
	d_GraphRect.setLeft(d_VLegendRect.right());
	d_GraphRect.setTop(d_TitleRect.bottom());
	d_GraphRect.setRight(X - d_LineHeight);
	d_GraphRect.setBottom(d_HLegendRect.top());
	
	d_VLegendTitleRect.setTop(d_GraphRect.top());
	d_VLegendTitleRect.setBottom(d_GraphRect.bottom());
	
	d_HLegendTitleRect.setLeft(d_GraphRect.left());
	d_HLegendTitleRect.setRight(d_GraphRect.right());
	
	this->DrawPixmap(Device);
	
	d_ViewportMat = OrgViewportMat;
	d_GraphRect = OrgGraphRect;
	d_VLegendRect = OrgVLegendRect;
	d_VLegendRect = OrgVLegendTitleRect;
	d_HLegendRect = OrgHLegendRect;
	d_HLegendTitleRect = OrgHLegendTitleRect;
	d_TitleRect = OrgTitleRect;
	d_Viewport= OrgViewport;
	d_LineHeight = OrgLineHeight;
	
	d_VLegendTitle = OrigVLegendTitle;
	d_HLegendTitle = OrigHLegendTitle;
	d_Title = OrigTitle;
	d_draw_border = OrgDrawBorder;
	
	return true;
}

void TGraphWidget::SetScaleMinimumY(
	const double						yerr)
{
	if(yerr > 0)
	{
		d_y_minmax = yerr;
	}
	else
	{
		d_y_minmax = 0.00005;
	}
}

void TGraphWidget::LockAspectRatio(const bool state)
{
	d_lock_aspect = state;
}

bool TGraphWidget::GenerateRGBColor(
	const int 							Set,
	int 								*R,
	int 								*G,
	int 								*B)
{
    int                           		DivOffset;
    int                           		Multiplier;
    int                           		TableOffset;
    static int Table[] =
    {
        1,0,0,
        0,1,0,
        0,0,1,
        1,1,0,
        0,1,1,
        1,0,1
    };
    static int DivTable[] =
    {
    	255,128,192,64,160,224,96,32,144,112,176,80,208,48,240,16
    };   // 16 colors. Max colors = 16 * 6 = 96
    
	if(Set > 96)
	{
		*R = 0;
		*G = 0;
		*B = 0;

		return false;
	}
     
     DivOffset = Set / 6;

     if(DivOffset > 15)
     {
          Multiplier = 0;
     }
     else
     {
          Multiplier = DivTable[DivOffset];
     }

     TableOffset = Set - (DivOffset * 6);
     TableOffset *= 3;

     *R = Table[TableOffset] * Multiplier;
     *G = (Table[TableOffset + 1] * Multiplier)/2;
     *B = Table[TableOffset + 2] * Multiplier;

     return true;
}
	
void TGraphWidget::ZoomOut(void)
{
	d_ZoomOutOnPaint = true;
	
	this->update();
}

void TGraphWidget::ScaleToFit(void)
{
	d_ScaleToFitOnPaint = true;
	this->update();
}

void TGraphWidget::paintEvent(
	QPaintEvent 						*event)
{
    QPainter 							Paint(this);
	QColor								Color(Qt::lightGray);
	QPen								OldPen;
	QRect								CoordRect;
	QRect								TextRect;
    QVector<QRect>						Rects = event->region().rects();
	QSize								Size;
	QRect								Rect;
	QPoint								Center;
    
    if(d_ScaleToFitOnPaint)
    {
    	d_ScaleToFitOnPaint = false;
    	
		this->DrawPixmap();
		update(d_Viewport.left(),
					d_Viewport.top(),
					d_Viewport.width(),
					d_Viewport.height());
    		
    	return;
    }
    
    if(d_ZoomOutOnPaint)
    {
    	d_ZoomOutOnPaint = false;
    	
	
		Size = d_GraphRect.size();
		Size *= 2;
	
		Center = d_GraphRect.center();
		Rect.setWidth(Size.width());
		Rect.setHeight(Size.height());
	
		Rect.moveCenter(Center);
	
		this->DrawPixmap(Rect,false);
		
		update(d_Viewport.left(),
					d_Viewport.top(),
					d_Viewport.width(),
					d_Viewport.height());
    	
    	return;
    }

    for(int i=0;i < static_cast<int>(Rects.size());++i)
    {
		Paint.drawPixmap(Rects[i],d_Pixmap,Rects[i]);
    }

    if(d_ShowMouseDownBox)
    {
		Paint.drawRect(d_MouseDownBox.normalized());
    }
    
	OldPen = Paint.pen();

	Paint.setPen(Color);
	
	TextRect = Paint.boundingRect(0,0,0,0,0,d_CoordDisplay);
	
	CoordRect.setLeft(d_GraphRect.right() - TextRect.width());
	CoordRect.setTop(d_GraphRect.top());
	CoordRect.setRight(d_GraphRect.right());
	CoordRect.setBottom(d_GraphRect.top() + TextRect.height());
	
	Paint.drawText(CoordRect.left(),CoordRect.bottom(),d_CoordDisplay);

	Paint.setPen(OldPen);
}

void TGraphWidget::resizeEvent(
	QResizeEvent 						*Size)
{
	int									X,Y;
	
	X = Size->size().width();
	Y = Size->size().height();

	d_Viewport.setLeft(0);
	d_Viewport.setTop(0);
	d_Viewport.setRight(X);
	d_Viewport.setBottom(Y);

	d_VLegendTitleRect.setLeft(d_Viewport.left());
	d_VLegendTitleRect.setTop(d_Viewport.top());
	d_VLegendTitleRect.setRight(d_LineHeight * 2 + d_Viewport.left());
	d_VLegendTitleRect.setBottom(Y - d_LineHeight * 2);

	d_HLegendTitleRect.setLeft(d_LineHeight * 2 + d_Viewport.left());
	d_HLegendTitleRect.setTop(Y - d_LineHeight * 2);
	d_HLegendTitleRect.setRight(X);
	d_HLegendTitleRect.setBottom(Y);

	d_TitleRect.setLeft(d_VLegendTitleRect.right() + 5 * d_LineHeight);
	d_TitleRect.setTop(d_Viewport.top());
	d_TitleRect.setRight(X);
	d_TitleRect.setBottom(d_LineHeight * 2 + d_Viewport.top());

	d_VLegendRect.setLeft(d_VLegendTitleRect.right());
	d_VLegendRect.setTop(d_Viewport.top());
	d_VLegendRect.setRight(d_VLegendTitleRect.right() + 4 * d_LineHeight);
	d_VLegendRect.setBottom(d_HLegendTitleRect.top());

	d_HLegendRect.setLeft(d_VLegendRect.right());
	d_HLegendRect.setTop(d_HLegendTitleRect.top() - d_LineHeight * 2);
	d_HLegendRect.setRight(X);
	d_HLegendRect.setBottom(d_HLegendTitleRect.top());

	d_GraphRect.setLeft(d_VLegendRect.right() + d_LineHeight);
	d_GraphRect.setTop(d_TitleRect.bottom());
	d_GraphRect.setRight(X - d_LineHeight);
	d_GraphRect.setBottom(d_HLegendRect.top());

	d_VLegendTitleRect.setTop(d_GraphRect.top());
	d_VLegendTitleRect.setBottom(d_GraphRect.bottom());

	d_HLegendTitleRect.setLeft(d_GraphRect.left());
	d_HLegendTitleRect.setRight(d_GraphRect.right());
	
	this->DrawPixmap();
}

void TGraphWidget::mousePressEvent(
	QMouseEvent							*event)
{
	if(event->button() & Qt::LeftButton)
	{
		if(d_GraphRect.contains(event->pos()))
		{
			d_MouseDownBox.setTopLeft(QPoint(event->pos()));
			d_MouseDownBox.setBottomRight(QPoint(event->pos()));
		}
		else
		{
			d_MouseDownBox.setTopLeft(QPoint(d_GraphRect.topLeft()));
			d_MouseDownBox.setBottomRight(QPoint(d_GraphRect.topLeft()));
		}	
		
		d_LeftMouseDown = true;
		d_LeftMouseDownMoving = false;
	
	}
	
	if(event->button() & Qt::RightButton)
	{
		d_MouseDownBox.setTopLeft(event->pos());
		d_MouseDownBox.setBottomRight(event->pos());
		
		d_RightMouseDownMoving = false;
		d_RightMouseDown = true;
	
	}
}

void TGraphWidget::mouseMoveEvent(
	QMouseEvent							*event)
{
	QPoint								Point;
	double								dX,dY;
	QRect								CoordRect;

	Point = event->pos();
	
	if(event->buttons() & Qt::LeftButton)
	{
		if(d_LeftMouseDown)
		{
			if(d_LeftMouseDownMoving)
			{
				this->UpdateLeftDownBox();
			}
			else
			{
				d_LeftMouseDownMoving = true;				
				d_ShowMouseDownBox = true;
			}
			
			d_MouseDownBox.setBottomRight(Point);
			this->UpdateLeftDownBox();
		}
	
	} 
	else if(event->buttons() & Qt::RightButton)
	{
		if(d_RightMouseDown)
		{
			d_MouseDownBox.setBottomRight(Point);
			
			CoordRect = d_GraphRect;
			
			CoordRect.adjust(
					d_MouseDownBox.left() - d_MouseDownBox.right(),
					d_MouseDownBox.top() - d_MouseDownBox.bottom(),
					d_MouseDownBox.left() - d_MouseDownBox.right(),
					d_MouseDownBox.top() - d_MouseDownBox.bottom());
					
			this->DrawPixmap(CoordRect,false);
			this->repaint();
	
			d_MouseDownBox.setTopLeft(Point);
		}
			
		d_RightMouseDownMoving = true;
	}
	else if(d_GraphRect.contains(Point))
	{
		d_ViewportMat.TranslateFromWin(Point.x(),Point.y(),&dX,&dY);
		
		d_CoordDisplay.sprintf("%.5lf,   %.5lf",dX,dY);
		
		this->repaint();
	}
}

void TGraphWidget::mouseReleaseEvent(
	QMouseEvent							*event)
{
	QPoint								Point;

	Point = event->pos();

	if(event->button() & Qt::LeftButton)
	{
		if(d_LeftMouseDownMoving)
		{
			d_ShowMouseDownBox = false;
			this->UpdateLeftDownBox();

			this->DrawPixmap(d_MouseDownBox,false);
				
			update(d_Viewport.left(),
						d_Viewport.top(),
						d_Viewport.width(),
						d_Viewport.height());
						
			d_LeftMouseDownMoving = false;						
		}
		
		d_LeftMouseDown = false;
	}
	
	if(event->button() & Qt::RightButton)
	{
		if(d_RightMouseDownMoving)
		{
			d_RightMouseDownMoving = false;						
		}
		else
		{
			this->ZoomOut();
		}
		
		d_RightMouseDown = false;
	}
}

void TGraphWidget::DrawPixmap(void)
{
	if(d_Viewport.height() < 1 || d_Viewport.width() < 1)
	{
		return;
	}
	
	d_Pixmap = QPixmap(QSize(d_Viewport.width(),d_Viewport.height()));
	d_Pixmap.fill(Qt::white);
	
	QPainter Painter(&d_Pixmap);

    Painter.setViewport(d_Viewport.left(),
    			d_Viewport.top(),
    			d_Viewport.right(),
    			d_Viewport.bottom());
	
	this->CalculateViewport();
    this->CalculateLegendFormat(&Painter);
	this->DrawLegend(&Painter);
	this->DrawGraph(&Painter);
}

void TGraphWidget::DrawPixmap(
	const QRect							&Rect,
	const bool							Resize)
{

	if(Resize)
	{
		d_Pixmap = QPixmap(QSize(d_Viewport.width(),d_Viewport.height()));
	}
	
	if(d_Pixmap.height() < 1 || d_Pixmap.width() < 1)
	{
		return;
	}
	
	d_Pixmap.fill(Qt::white);

	QPainter Painter(&d_Pixmap);

    Painter.setViewport(d_Viewport.left(),
    			d_Viewport.top(),
    			d_Viewport.right(),
    			d_Viewport.bottom());
	
	this->CalculateViewport(Rect);
    this->CalculateLegendFormat(&Painter);
	this->DrawLegend(&Painter);
	this->DrawGraph(&Painter);
}

void TGraphWidget::DrawPixmap(
	QPainter							*Device)
{
	this->CalculateViewport();
    this->CalculateLegendFormat(Device);
	this->DrawLegend(Device);
	this->DrawGraph(Device);
}


void TGraphWidget::CalculateViewport(void)
{
	std::vector<TDataSet>::iterator     iter_set;
	std::vector<TVector2>::iterator		iter_pnt;
    double                              MinX,MaxX,MinY,MaxY;
    TViewportMat                        W,S,V;
    bool                                init;
	double								dvalx;//,dvaly;
	double								dsclx,dscly;

 	MaxX = 0.000005;
    MinX = -0.000005;
    MaxY = 0.000005;
    MinY = -0.000005;
 
	init = true;

	for(iter_set = d_DataSets.begin();iter_set != d_DataSets.end();++iter_set)
	{
		for(iter_pnt = (*iter_set).Points.begin();iter_pnt != (*iter_set).Points.end();++iter_pnt)
		{
			if(init)
			{
				MinX = MaxX = (*iter_pnt).x;
				MinY = MaxY = (*iter_pnt).y;

				init = false;
			}
			else
			{
				if((*iter_pnt).x > MaxX)
				{
					MaxX = (*iter_pnt).x;
				}

				if((*iter_pnt).x < MinX)
				{
					MinX = (*iter_pnt).x;
				}

				if((*iter_pnt).y > MaxY)
				{
					MaxY = (*iter_pnt).y;
				}

				if((*iter_pnt).y < MinY)
				{
					MinY = (*iter_pnt).y;
				}
			}
		}
	}
	
	if(MinY > (-1 * d_y_minmax))
	{
		MinY = -1 * d_y_minmax;
	}
	
	if(MaxY < d_y_minmax)
	{
		MaxY = d_y_minmax;
	}

	if(d_GraphType == TGraphWidget::BAR_GRAPH)
	{
		dvalx = (MaxX - MinX) / 20;
		MaxX += dvalx;
		MinX -= dvalx;
	}

    if((MaxX - MinX) < 0.00001)
    {
    	MaxX+= 0.000005;
        MinX-= 0.000005;
    }
	
	// set x axis scale same as y axis
	if(d_lock_aspect)
	{
		dvalx = (MaxX + MinX)/2.0;
//		dvaly = (MaxY + MinY)/2.0;
		
		dscly = abs(d_GraphRect.top() - d_GraphRect.bottom());
		dsclx = abs(d_GraphRect.right() - d_GraphRect.left());
		
		if(dscly > 0)
		{
			dsclx = dsclx / dscly;
		}
		else
		{
			dsclx = 1.0;
		}
		
		MaxX = dvalx + MaxY * dsclx;
		MinX = dvalx + MinY * dsclx;
	}

    if((MaxY - MinY) < 0.00001)
    {
    	MaxY += 0.000005;
        MinY -= 0.000005;
    }

    d_RangeX = MaxX - MinX;
    d_RangeY = MaxY - MinY;


	if(d_VLegendMode == TGraphWidget::DEFAULT_LEGEND_PRECISION)
    {
		MaxY += (d_RangeY / 50);
		MinY -= (d_RangeY / 50);
    }

	if(d_HLegendMode == TGraphWidget::DEFAULT_LEGEND_PRECISION)
    {
		MaxX += (d_RangeX / 50);
		MinX -= (d_RangeX / 50);
    }

	W.tx[0] = 1;
    W.tx[1] = 0;
    W.ty[0] = 0;
    W.ty[1] = 1;
    W.to[0] = -MinX;
    W.to[1] = -MinY;

    S.tx[0] = (d_GraphRect.right() - d_GraphRect.left()) / (MaxX - MinX);
    S.tx[1] = 0;
    S.ty[0] = 0;
    S.ty[1] = (d_GraphRect.top() - d_GraphRect.bottom()) / (MaxY - MinY);
    S.to[0] = 0;
    S.to[1] = 0;

    V.tx[0] = 1;
    V.tx[1] = 0;
    V.ty[0] = 0;
    V.ty[1] = 1;
    V.to[0] = d_GraphRect.left();
    V.to[1] = d_GraphRect.bottom();

    d_ViewportMat = W;
    d_ViewportMat *=S;
    d_ViewportMat *=V;    
}

void TGraphWidget::CalculateViewport(
	const QRect							&Rect)
{
    double                              MinX,MaxX,MinY,MaxY;
    double								ScaleLimit;
    TViewportMat                        W,S,V;
	QRect								WinRect;

	WinRect = Rect.normalized();

	d_ViewportMat.TranslateFromWin(WinRect.left(),WinRect.top(),&MinX,&MaxY);
	d_ViewportMat.TranslateFromWin(WinRect.right(),WinRect.bottom(),&MaxX,&MinY);

	if((MaxX - MinX) > (d_RangeX * 50))
	{
		ScaleLimit = (d_RangeX * 50) / (MaxX - MinX);

		MaxX *= ScaleLimit;
		MinX *= ScaleLimit;
	}

	if((MaxY - MinY) > (d_RangeY * 50))
	{
		ScaleLimit = (d_RangeY * 50) / (MaxY - MinY);

		MaxY *= ScaleLimit;
		MinY *= ScaleLimit;
	}

    if((MaxX - MinX) < 0.00001)
    {
    	MaxX+= 0.000005;
        MinX-= 0.000005;
    }

    if((MaxY - MinY) < 0.00001)
    {
    	MaxY += 0.000005;
        MinY -= 0.000005;
    }


	W.tx[0] = 1;
    W.tx[1] = 0;
    W.ty[0] = 0;
    W.ty[1] = 1;
    W.to[0] = -MinX;
    W.to[1] = -MinY;

    S.tx[0] = (d_GraphRect.right() - d_GraphRect.left()) / (MaxX - MinX);
    S.tx[1] = 0;
    S.ty[0] = 0;
    S.ty[1] = (d_GraphRect.top() - d_GraphRect.bottom()) / (MaxY - MinY);
    S.to[0] = 0;
    S.to[1] = 0;

    V.tx[0] = 1;
    V.tx[1] = 0;
    V.ty[0] = 0;
    V.ty[1] = 1;
    V.to[0] = d_GraphRect.left();
    V.to[1] = d_GraphRect.bottom();

    d_ViewportMat = W;
    d_ViewportMat *=S;
    d_ViewportMat *=V;
}

void TGraphWidget::DrawLegend(
	QPainter							*Painter)
{
	QFont								OldFont;
	QFont								Font;
	QRect								BoundingRect;
	int                           		NumberItems;
    int                           		AdjTextWidth;
    QPoint                         		TextOffset;
    QPoint                         		TextPoint;
    double                        		Scale;
    double                        		Xd,Yd;
    int									Xi,Yi;
    int                           		Cntr;
    QString                       		Text;
    QPen								OldPen;
    QPen								BorderPen;
    int									YMax;
    int									YMin;
    int									YLast(0);
    bool								Init;

	OldPen = Painter->pen();
	OldFont = Painter->font();
	
	QFontMetrics						FontMetrics(OldFont);
	
// Horizontal Axis
	
//    TextOffset.setY(static_cast<int>(static_cast<double>(d_HMaxLabelSize.width()) * 0.707107));
//    TextOffset.setX(TextOffset.y());
    TextOffset.setY(d_HMaxLabelSize.height());
    TextOffset.setX(d_HMaxLabelSize.width()/2);

//    AdjTextWidth = static_cast<int>(static_cast<double>(d_HMaxLabelSize.height()) / 0.707107);
    AdjTextWidth = d_HMaxLabelSize.width();

    NumberItems = d_GraphRect.width() / AdjTextWidth;

    Scale = static_cast<double>(d_GraphRect.width()) /
               static_cast<double>(NumberItems - 1);

    for(Cntr = 0;Cntr < NumberItems;++Cntr)
    {
    	TextPoint.setX(d_GraphRect.left() + static_cast<int>(Cntr * Scale));

		Painter->setPen(QColor(Qt::lightGray));
//        Painter->drawLine(TextPoint.x(),
//        			d_GraphRect.top(),
//        			TextPoint.x(),
//        			d_GraphRect.bottom());
        			
        Painter->drawLine(TextPoint.x(),
        			d_GraphRect.bottom(),
        			TextPoint.x(),//-FontMetrics.height() / 3,
        			d_GraphRect.bottom()+FontMetrics.height()/3);

        d_ViewportMat.TranslateFromWin(TextPoint.x(),0,&Xd,&Yd);

        TextPoint.setY(d_HLegendRect.top() + FontMetrics.height());
        
        Text.sprintf(d_HFormat.toLatin1(),Xd);
        
        
        if(d_HLegendMode != TGraphWidget::NO_LEGEND)
        {
			Painter->save();
			Painter->translate(
					static_cast<double>(TextPoint.x() - TextOffset.x()),
					static_cast<double>(TextPoint.y() + TextOffset.y()));
					
//			Painter->rotate(-45);
	
			Painter->setPen(QColor(Qt::black));
	
			Painter->drawText(
						0,
						-d_HMaxLabelSize.height() / 2,
						Text);
			
			Painter->restore();
        }
	}
	
// Vertical Axis

	TextOffset.setX(d_VMaxLabelSize.width());
    TextOffset.setY(d_VMaxLabelSize.height()/2);

    if(d_VLegendMode == TGraphWidget::DEFAULT_LEGEND_PRECISION)
    {
		NumberItems = d_GraphRect.height() / d_LineHeight;

		Scale = static_cast<double>(d_GraphRect.height()) /
				   static_cast<double>(NumberItems - 1);

		for(Cntr = 0;Cntr < NumberItems;++Cntr)
		{
			TextPoint.setY(d_GraphRect.top() + static_cast<int>(Cntr * Scale));

			Painter->setPen(QColor(Qt::lightGray));
			Painter->drawLine(d_GraphRect.left()-FontMetrics.height() / 2,
						TextPoint.y(),
						d_GraphRect.left(),//d_GraphRect.right(),
						TextPoint.y());

			d_ViewportMat.TranslateFromWin(0,TextPoint.y(),&Xd,&Yd);

			TextPoint.setX(d_VLegendRect.right() - d_VMaxLabelSize.width());
			TextPoint.setY(TextPoint.y() + d_VMaxLabelSize.height()/2 - 1);

			Text.sprintf(d_VFormat.toLatin1(),Yd);

			Painter->setPen(QColor(Qt::black));
			Painter->drawText(TextPoint.x(),TextPoint.y(),Text);
		}
    }
    else if(d_VLegendMode == TGraphWidget::INTEGER_LEGEND_PRECISION)
    {
    	// Find range of integer values that fit inside graph
		d_ViewportMat.TranslateFromWin(0,d_GraphRect.height() + d_GraphRect.top(),&Xd,&Yd);
		YMin = Yd;
		d_ViewportMat.TranslateFromWin(0,d_GraphRect.top(),&Xd,&Yd);
		YMax = Yd;

		Init = true;

		for(Cntr = YMin;Cntr <= YMax;Cntr++)
		{
			d_ViewportMat.TranslateToWin(0,static_cast<double>(Cntr),&Xi,&Yi);

			TextPoint.setY(Yi);

			if(abs(Yi - YLast) > d_VMaxLabelSize.height() || Init == true)
			{
				Init = false;

				Painter->setPen(QColor(Qt::lightGray));
				Painter->drawLine(d_GraphRect.left()-FontMetrics.height() / 2,
							TextPoint.y(),
							d_GraphRect.left(),//d_GraphRect.right(),
							TextPoint.y());

				TextPoint.setX(d_VLegendRect.right() - d_VMaxLabelSize.width()/2);
				TextPoint.setY(TextPoint.y() + d_VMaxLabelSize.height()/2 - 1);

				Text = QString("%1").arg(Cntr);

				Painter->setPen(QColor(Qt::black));
				Painter->drawText(TextPoint.x(),TextPoint.y(),Text);

				YLast = Yi;
			}
		}
    }

// Draw a line at zero graph zero if within the graph viewport
	Xd = 0;
	Yd = 0;
	d_ViewportMat.TranslateToWin(Xd,Yd,&Xi,&Yi);
	
//	Painter->setPen(QColor(Qt::blue));
	Painter->setPen(QColor(Qt::lightGray));
	
	if(Xi > d_GraphRect.left() && Xi < d_GraphRect.right())
	{
		Painter->drawLine(
					Xi,
					d_GraphRect.top(),
					Xi,
					d_GraphRect.bottom());
	}
	
	if(Yi > d_GraphRect.top() && Yi < d_GraphRect.bottom())
	{
		Painter->drawLine(
					d_GraphRect.left(),
					Yi,
					d_GraphRect.right(),
					Yi);
	}
	
// This is the border around the graph widget.

	if(d_draw_border)
	{
		Painter->setPen(QColor(Qt::darkGray));
		Painter->drawRect(0,0,d_Viewport.width()-1,d_Viewport.height()-1);
	}
	
	
// This is the border around the graph.
	Painter->setPen(QColor(Qt::darkGray));
	Painter->drawRect(d_GraphRect.left(),d_GraphRect.top(),d_GraphRect.width(),d_GraphRect.height());

// Shadow border
//	BorderPen.setWidth(2);
//	BorderPen.setColor(QColor(Qt::darkGray));
//	Painter->setPen(BorderPen);
	
//	Painter->drawLine(
//				d_GraphRect.right()+2,
//				d_GraphRect.top() + 1,
//				d_GraphRect.right()+2,
//				d_GraphRect.bottom());
					
//	Painter->drawLine(
//				d_GraphRect.left() + 1,
//				d_GraphRect.bottom() + 2,
//				d_GraphRect.right() + 2,
//				d_GraphRect.bottom() + 2);

    Painter->setPen(OldPen);

// Title    
	Font.setPointSize(12);
	Font.setBold(true);
	Painter->setFont(Font);

	Painter->drawText(d_TitleRect,Qt::AlignCenter,d_Title);

	Font.setPointSize(10);
	Painter->setFont(Font);

	Painter->drawText(d_HLegendTitleRect,Qt::AlignCenter,d_HLegendTitle);

	BoundingRect = Painter->boundingRect(0,0,0,0,Qt::AlignLeft,d_VLegendTitle);
	
	if(BoundingRect.width() < 1)
	{
		BoundingRect.setWidth(1);
	}
	
	if(BoundingRect.height() < 1)
	{
		BoundingRect.setHeight(1);
	}

	
	Painter->save();
	Painter->translate(
		static_cast<double>(d_VLegendTitleRect.center().x()),
		static_cast<double>(d_VLegendTitleRect.center().y()));
		
	Painter->rotate(-90);

	Painter->drawText(-BoundingRect.width()/2,0,d_VLegendTitle);
	Painter->restore();

	Painter->setFont(OldFont);
}

void TGraphWidget::DrawGraph(
	QPainter							*Painter)
{
    int									X,Y,X2,Y2,LastX,LastY;
    int									BarWidth;
	QPen								OldPen;
	bool								FirstIteration;
	std::vector<TDataSet>::const_iterator	set_iter;
	std::vector<TVector2>::const_iterator	pnt_iter;

	OldPen = Painter->pen();
	
	if(!d_DataSets.empty())
    {
		Painter->setClipRect(d_GraphRect);
		
		for(set_iter = d_DataSets.begin();set_iter != d_DataSets.end();set_iter++)
		{
			if((*set_iter).Points.size())
			{
				if((d_GraphType == TGraphWidget::BAR_GRAPH && (*set_iter).DatasetGraphType == TGraphWidget::DEFAULT_GRAPH) ||
				   (*set_iter).DatasetGraphType == TGraphWidget::BAR_GRAPH)
				{
					Painter->setPen((*set_iter).Color);
					
					BarWidth = d_GraphRect.width();
					BarWidth /= (static_cast<int>((*set_iter).Points.size()) * 2);
					
					BarWidth -= 1;
					// make sure at least one pixel between bars
					if(BarWidth < 1)
					{
						BarWidth = 1;
					}
					
					for(pnt_iter = (*set_iter).Points.begin();pnt_iter != (*set_iter).Points.end();pnt_iter++)
					{
						d_ViewportMat.TranslateToWin((*pnt_iter).x,(*pnt_iter).y,&X,&Y);
						d_ViewportMat.TranslateToWin((*pnt_iter).x,0.0,&X2,&Y2);
						
						Painter->fillRect(X - BarWidth,Y,2 * BarWidth,Y2 - Y,(*set_iter).Color);
					}
				}
				else if((d_GraphType == TGraphWidget::LINE_GRAPH && (*set_iter).DatasetGraphType == TGraphWidget::DEFAULT_GRAPH) ||
				   (*set_iter).DatasetGraphType == TGraphWidget::LINE_GRAPH ||
				   d_GraphType == TGraphWidget::DEFAULT_GRAPH)
				{
					Painter->setPen((*set_iter).Color);
					
					FirstIteration = true;
					for(pnt_iter = (*set_iter).Points.begin();pnt_iter != (*set_iter).Points.end();pnt_iter++)
					{
						if(FirstIteration)
						{
							d_ViewportMat.TranslateToWin((*pnt_iter).x,(*pnt_iter).y,&LastX,&LastY);
							FirstIteration = false;
						}
						else
						{
							d_ViewportMat.TranslateToWin((*pnt_iter).x,(*pnt_iter).y,&X,&Y);
							
							Painter->drawLine(LastX,LastY,X,Y);
							LastX = X;
							LastY = Y;
						}
					}
				}
				else if((d_GraphType == TGraphWidget::POINT_GRAPH && (*set_iter).DatasetGraphType == TGraphWidget::DEFAULT_GRAPH) ||
						(*set_iter).DatasetGraphType == TGraphWidget::POINT_GRAPH)
				{						
					Painter->setPen((*set_iter).Color);
					
					for(pnt_iter = (*set_iter).Points.begin();pnt_iter != (*set_iter).Points.end();pnt_iter++)
					{
						d_ViewportMat.TranslateToWin((*pnt_iter).x,(*pnt_iter).y,&X,&Y);
						
						Painter->drawLine(X-3,Y-3,X+3,Y+3);
						Painter->drawLine(X-3,Y+3,X+3,Y-3);
					}
				}
			}
		}
		
		Painter->setClipping(false);
	}
	
	Painter->setPen(OldPen);
}

void TGraphWidget::CalculateLegendFormat(
	QPainter							*Painter)
{
    double								MaxX,MaxY,MinX,MinY;
    double								RefX,RefY;
    double								RangeX,RangeY;
    int									Total;
    int									Trailing;
    QString								LabelField;
    QString								Text;
    QRect								BoundingRect;

    d_ViewportMat.TranslateFromWin(
    			d_GraphRect.left(),
				d_GraphRect.top(),
				&MinX,
				&MaxY);

	d_ViewportMat.TranslateFromWin(
				d_GraphRect.right(),
				d_GraphRect.bottom(),
				&MaxX,
				&MinY);

	if(fabs(MaxX) > fabs(MinX))
    {
		RefX = fabs(MaxX);
	}
    else
    {
    	RefX = fabs(MinX);
    }

    if(fabs(MaxY) > fabs(MinY))
    {
    	RefY = fabs(MaxY);
    }
    else
    {
    	RefY = fabs(MinY);
    }

    RangeX = MaxX - MinX;
    RangeY = MaxY - MinY;

    if(RangeX < 1)
    {
    	RangeX = 1;
    }

    if(RangeY < 1)
    {
    	RangeY = 1;
    }

    if(d_HLegendMode == TGraphWidget::DEFAULT_LEGEND_PRECISION)
    {
		if(RefX > 1)
		{
			Total = static_cast<int>(log10(RefX));
			Trailing = 5 - static_cast<int>(log(RangeX));

			if(Trailing < 0)
			{
				Trailing = 0;
			}
		}
		else
		{
			Total = 0;
			Trailing = 5;
		}
    }
    else
    {
		Total = 5;
		Trailing = 0;
    }

    Total = Total + Trailing + 1;

    d_HFormat.sprintf("%% %d.%dlf",
				Total,
				Trailing);

	LabelField.sprintf("%%0%d.%dlf  ",
				Total,
				Trailing);
				
	Text.sprintf(LabelField.toLatin1(),static_cast<double>(0.0));

	BoundingRect = Painter->boundingRect(0,0,0,0,0,Text);

	d_HMaxLabelSize = QSize(BoundingRect.width(),BoundingRect.height());
	
	if(d_HMaxLabelSize.width() < 1)
	{
		d_HMaxLabelSize.setWidth(1);
	}
	
	if(d_HMaxLabelSize.height() < 1)
	{
		d_HMaxLabelSize.setHeight(1);
	}
    
    if(d_VLegendMode == TGraphWidget::DEFAULT_LEGEND_PRECISION)
    {
		if(RefY > 1)
		{
			Total = static_cast<int>(log10(RefY));
			Trailing = 5 - static_cast<int>(log(RangeY));

			if(Trailing < 0)
			{
				Trailing = 0;
			}
		}
	    else
	    {
			Total = 0;
			Trailing = 5;
		}
    }
    else
    {
		Total = 5;
		Trailing = 0;
	}

	Total = Total + Trailing + 1;

    d_VFormat.sprintf("%% %d.%dlf",
				Total,
				Trailing);

 	LabelField.sprintf("%%0%d.%dlf    ",
				Total,
				Trailing);

	Text.sprintf(LabelField.toLatin1(),static_cast<double>(0.0));

	BoundingRect = Painter->boundingRect(0,0,0,0,0,Text);
	d_VMaxLabelSize = QSize(BoundingRect.width(),BoundingRect.height());
	
	if(d_VMaxLabelSize.width() < 1)
	{
		d_VMaxLabelSize.setWidth(1);
	}
	
	if(d_VMaxLabelSize.height() < 1)
	{
		d_VMaxLabelSize.setHeight(1);
	}
}

void TGraphWidget::UpdateLeftDownBox(void)
{
	QRect	Rect = d_MouseDownBox.normalized();

	update(Rect.left(),Rect.top(),Rect.width(),1);
	update(Rect.left(),Rect.top(),1,Rect.height());
	update(Rect.left(),Rect.bottom()+1,Rect.width(),1);
	update(Rect.right()+1,Rect.top(),1,Rect.height());
}



