/////////////////////////////////////////////////////////////////////
//
//            X   X           X
//           XXX XX         XX
//          XXXXXXXX      XXX
//         XX X XXXXXXXXXXX
//        XXXXX XXXXXXXXX
//       XXXXX XXXXXXXXXX
//            XXX XXX XXX
//           XXX XX   XX
//           X   X     X
//
//    Copyright (C) 2003-2016  Ron Jakl
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
/////////////////////////////////////////////////////////////////////

#include <QApplication>
#include <QDockWidget>
#include <QFont>
#include <QGroupBox>
#include <QLabel>
#include <QPlainTextEdit>
#include <QPushButton>
#include <QSpacerItem>
#include <QVBoxLayout>
#include <QHBoxLayout>
#include <QWidget>
#include <QSplitter>
#include <QColor>
#include <QFileDialog>
#include <QToolButton>
#include <QDragEnterEvent>
#include <QDropEvent>
#include <QMimeData>
#include <QUrl>
#include <QClipboard>
#include <QDir>
#include <QList>

#include "../../core/graphwidget.h"

#include "laserdataviewer.h"

TLaserDataViewer::TLaserDataViewer(
	const QWidget						*parent,
	const Qt::WindowFlags				flags)
:QMainWindow(const_cast<QWidget*>(parent),flags)
{
	QDockWidget							*control_dock;
	QWidget								*control_dock_widget;
	QFont								font;
	QFont								copyright_font;
	QGroupBox							*clipboard_group;
	QGroupBox							*file_group;
	QGroupBox							*section_group;
	QLabel								*copyright_label;
	QPushButton							*close_button;
	QPushButton							*import_button;
	QSpacerItem							*control_dock_vspacer;
	QVBoxLayout							*central_vlayout;
	QVBoxLayout							*clipboard_group_vlayout;
	QVBoxLayout							*control_dock_widget_vlayout;
	QVBoxLayout							*file_group_vlayout;
	QHBoxLayout							*section_group_hlayout;
	QWidget								*centralwidget;
	QSplitter							*splitter_widget;
	QList<int>							splitter_sizes;

	font.setFamily(QStringLiteral("Courier New"));

	copyright_font.setPointSize(8);
	copyright_font.setItalic(true);
	
	// set size of main window
	// set initial height of splitter widgets
	// want to have view at 50/50
	this->resize(801,601);
	splitter_sizes.push_back(300);
	splitter_sizes.push_back(300);
	
	this->setAcceptDrops(true);

	centralwidget = new QWidget(this);
	this->setCentralWidget(centralwidget);

	central_vlayout = new QVBoxLayout(centralwidget);
	
	splitter_widget = new QSplitter(Qt::Vertical,centralwidget);

	d_data_text = new QPlainTextEdit;
	d_data_text->setFont(font);
	d_data_text->setUndoRedoEnabled(false);
	d_data_text->setLineWrapMode(QPlainTextEdit::NoWrap);
	d_data_text->setReadOnly(true);
	splitter_widget->addWidget(d_data_text);
	
	d_graph_display_widget = new TGraphWidget;
	d_graph_display_widget->setMinimumSize(0,150);
	splitter_widget->addWidget(d_graph_display_widget);

	central_vlayout->addWidget(splitter_widget);

	copyright_label = new QLabel(centralwidget);
	copyright_label->setFont(copyright_font);
	central_vlayout->addWidget(copyright_label);

	control_dock = new QDockWidget(this);
	this->addDockWidget(static_cast<Qt::DockWidgetArea>(1),control_dock);

	control_dock_widget = new QWidget();
	control_dock->setWidget(control_dock_widget);

	control_dock_widget_vlayout = new QVBoxLayout(control_dock_widget);

	file_group = new QGroupBox(control_dock_widget);

	file_group_vlayout = new QVBoxLayout(file_group);
	file_group_vlayout->setContentsMargins(2,-1,2,-1);

	import_button = new QPushButton(file_group);
	file_group_vlayout->addWidget(import_button);

	d_saveas_button = new QPushButton(file_group);
	file_group_vlayout->addWidget(d_saveas_button);
	control_dock_widget_vlayout->addWidget(file_group);

	section_group = new QGroupBox(control_dock_widget);
	
	section_group_hlayout = new QHBoxLayout(section_group);
	section_group_hlayout->setContentsMargins(2,-1,2,-1);
	
	d_section_1_button = new QToolButton(section_group);
	d_section_1_button->setCheckable(true);
	section_group_hlayout->addWidget(d_section_1_button);
	
	d_section_2_button = new QToolButton(section_group);
	d_section_2_button->setCheckable(true);
	section_group_hlayout->addWidget(d_section_2_button);
	
	d_section_3_button = new QToolButton(section_group);
	d_section_3_button->setCheckable(true);
	section_group_hlayout->addWidget(d_section_3_button);

	control_dock_widget_vlayout->addWidget(section_group);
	
	clipboard_group = new QGroupBox(control_dock_widget);

	clipboard_group_vlayout = new QVBoxLayout(clipboard_group);
	clipboard_group_vlayout->setContentsMargins(2,-1,2,-1);

	d_copy_normal_button = new QPushButton(clipboard_group);
	clipboard_group_vlayout->addWidget(d_copy_normal_button);

	d_copy_inverted_button = new QPushButton(clipboard_group);
	clipboard_group_vlayout->addWidget(d_copy_inverted_button);
	
	d_copy_all_button = new QPushButton(clipboard_group);
	clipboard_group_vlayout->addWidget(d_copy_all_button);
	
	control_dock_widget_vlayout->addWidget(clipboard_group);

	control_dock_vspacer = new QSpacerItem(0,0,QSizePolicy::Minimum,QSizePolicy::Expanding);
	control_dock_widget_vlayout->addItem(control_dock_vspacer);

	close_button = new QPushButton(control_dock_widget);
	control_dock_widget_vlayout->addWidget(close_button);
	
	// maybe a bug with floatable dock widgets on OSX ?
#ifdef Q_OS_MAC
	control_dock->setFeatures(QDockWidget::DockWidgetMovable);
#else
	control_dock->setFeatures(QDockWidget::DockWidgetFloatable|QDockWidget::DockWidgetMovable);
#endif
	control_dock->setAllowedAreas(Qt::LeftDockWidgetArea|Qt::RightDockWidgetArea);
	
	QWidget::setTabOrder(import_button,d_saveas_button);
	QWidget::setTabOrder(d_saveas_button,d_section_1_button);
	QWidget::setTabOrder(d_section_1_button,d_section_2_button);
	QWidget::setTabOrder(d_section_2_button,d_section_3_button);
	QWidget::setTabOrder(d_section_3_button,d_copy_normal_button);
	QWidget::setTabOrder(d_copy_normal_button,d_copy_inverted_button);
	QWidget::setTabOrder(d_copy_inverted_button,d_copy_all_button);
	QWidget::setTabOrder(d_copy_all_button,d_data_text);

	this->setWindowTitle(QStringLiteral("Laser Data Viewer - 1.1"));
	this->setWindowIcon(QIcon(":/images/laserdataviewer.png"));

	copyright_label->setText(QStringLiteral("Laser Data Viewer (C) 2016 Select Calibration Incoprorated  www.selectcalibration.ca"));
	file_group->setTitle(QStringLiteral("File"));
	import_button->setText(QStringLiteral("Import"));
	d_saveas_button->setText(QStringLiteral("Save As..."));
	section_group->setTitle(QStringLiteral("Display Section"));
	d_section_1_button->setText(QStringLiteral("1"));
	d_section_2_button->setText(QStringLiteral("2"));
	d_section_3_button->setText(QStringLiteral("3"));
	clipboard_group->setTitle(QStringLiteral("Copy to Clipboard"));
	d_copy_normal_button->setText(QStringLiteral("Dev. Normal"));
	d_copy_inverted_button->setText(QStringLiteral("Dev. Inverted"));
	d_copy_all_button->setText(QStringLiteral("All Data"));
	close_button->setText(QStringLiteral("Close"));
	
	// defaults
	d_graph_display_widget->Title(QString());
	d_graph_display_widget->HorizontalLegend(QStringLiteral("Position"));
	d_graph_display_widget->VerticalLegend(QStringLiteral("Deviation"));
	d_section_1_button->setChecked(true);
	splitter_widget->setSizes(splitter_sizes);

	this->Enable_Options(false);
	
	connect(import_button,SIGNAL(clicked(void)),this,SLOT(Import_Data(void)));
	connect(d_saveas_button,SIGNAL(clicked(void)),this,SLOT(Save_As(void)));
	connect(d_copy_normal_button,SIGNAL(clicked(void)),this,SLOT(Copy_Dev_Normal(void)));
	connect(d_copy_inverted_button,SIGNAL(clicked(void)),this,SLOT(Copy_Dev_Invert(void)));
	connect(d_copy_all_button,SIGNAL(clicked(void)),this,SLOT(Copy_All_Data(void)));
	
	connect(d_section_1_button,SIGNAL(toggled(bool)),this,SLOT(Section_1_Toggled(bool)));
	connect(d_section_2_button,SIGNAL(toggled(bool)),this,SLOT(Section_2_Toggled(bool)));
	connect(d_section_3_button,SIGNAL(toggled(bool)),this,SLOT(Section_3_Toggled(bool)));

	connect(close_button,SIGNAL(clicked(void)),this,SLOT(close(void)));

}

TLaserDataViewer::~TLaserDataViewer(void)
{
}

void TLaserDataViewer::dragEnterEvent(
	QDragEnterEvent 					*drag_enter_event)
{
	if(drag_enter_event->mimeData()->hasFormat("text/uri-list"))
	{
		drag_enter_event->acceptProposedAction();
	}
}

void TLaserDataViewer::dropEvent(
	QDropEvent 							*drop_event)
{
	QList<QUrl>							urls;
	
	urls = drop_event->mimeData()->urls();
	
	if(urls.size() == 1)
	{
		this->Load_Data((*urls.begin()).toLocalFile());
	}
	else
	{
		this->Clear_Display();
		this->Clear_Graph();
		
		this->Add_Display_Text(QStringLiteral("ERR:  Only one input file is expected."));
	}
}

void TLaserDataViewer::Import_Data(void)
{
	QString								file_name;
	QString								file_filter;
	
	file_filter = QStringLiteral("Laser Files (*.rtl *.pos *.stx *.sty *.stz *.str *.rta *.ang)");
	
	file_name = QFileDialog::getOpenFileName(this,
											 QStringLiteral("Select Laser File to Load"),
											 d_laser_file.File_Name(),
											 file_filter);
	
	if(file_name.length())
	{
		this->Load_Data(file_name);
	}
}

void TLaserDataViewer::Save_As(void)
{
	QString								file_name;
	QString								suggested_file_name;
	QFile								file;
	QString								text;
	std::vector<TVector2>				data;
	std::vector<TVector2>::const_iterator iter;
	int									count;
	int									cntr;
	std::vector<QString>				text_lines;
	std::vector<QString>::const_iterator str_iter;
	int									index;
	int									active_section;
	
	active_section = this->Get_Active_Section();
	
	if(active_section < 0)
	{
		this->Add_Display_Text(QStringLiteral("ERR:  Data section not active."));
		return;
	}
	
	suggested_file_name = d_laser_file.File_Name();

	index = suggested_file_name.lastIndexOf('.');
	
	if(!(index<0))
	{
		suggested_file_name.truncate(index);
	}
	
	suggested_file_name += QString(" - %1.csv").arg(d_laser_file.Section_Name(active_section));

	file_name = QFileDialog::getSaveFileName(this,
											 QStringLiteral("Select file to write"),
											 suggested_file_name,
											 QStringLiteral("CSV Files (*.csv)"));
	
	if(file_name.length())
	{
		file.setFileName(file_name);
		
		if(!file.open(QIODevice::WriteOnly | QIODevice::Text))
		{
			this->Add_Display_Text(QString("ERR:  Cannot write output file '%1'").arg(QDir::toNativeSeparators(file_name)));
			return;
		}
		
		count = d_laser_file.Data_Count(active_section);
		
		for(cntr = 0;cntr < count;++cntr)
		{
			data = d_laser_file.Data(active_section,cntr);
			
			if(text_lines.size() < data.size())
			{
				text_lines.resize(data.size());
			}
			
			for(iter = data.begin(),index = 0;iter != data.end();++iter,++index)
			{
				if(cntr == 0)
				{
					text = QString("%1").arg((*iter).x,0,'f',5);
					text_lines[index] = text;
				}
				
				text = QString(",%1").arg((*iter).y,0,'f',5);
				
				text_lines[index] += text;
			}
		}
		
		for(str_iter = text_lines.begin();str_iter != text_lines.end();++str_iter)
		{
			file.write((*str_iter).toLatin1());
			file.write("\n");
		}
		
		file.close();
	}
}

void TLaserDataViewer::Copy_Dev_Normal(void)
{
	QString								text;
	std::vector<TVector2>				data;
	std::vector<TVector2>::const_iterator iter;
	int									active_section;
	
	active_section = this->Get_Active_Section();
	
	if(active_section < 0)
	{
		this->Add_Display_Text(QStringLiteral("ERR:  Data section not active."));
		return;
	}

	data = d_laser_file.Data(active_section,-1);
	
	for(iter = data.begin();iter != data.end();++iter)
	{
		text += QString("%1\n").arg((*iter).y,0,'f',5);
	}
	
	QApplication::clipboard()->setText(text,QClipboard::Clipboard);
}

void TLaserDataViewer::Copy_Dev_Invert(void)
{
	QString								text;
	std::vector<TVector2>				data;
	std::vector<TVector2>::const_iterator iter;
	int									active_section;
	
	active_section = this->Get_Active_Section();
	
	if(active_section < 0)
	{
		this->Add_Display_Text(QStringLiteral("ERR:  Data section not active."));
		return;
	}
	
	data = d_laser_file.Data(active_section,-1);
	
	for(iter = data.begin();iter != data.end();++iter)
	{
		text += QString("%1\n").arg(-1 * (*iter).y,0,'f',5);
	}
	
	QApplication::clipboard()->setText(text,QClipboard::Clipboard);
}

void TLaserDataViewer::Copy_All_Data(void)
{
	QString								text;
	std::vector<TVector2>				data;
	std::vector<TVector2>::const_iterator iter;
	int									count;
	int									cntr;
	std::vector<QString>				text_lines;
	std::vector<QString>::const_iterator str_iter;
	int									index;
	int									active_section;
	
	active_section = this->Get_Active_Section();
	
	if(active_section < 0)
	{
		this->Add_Display_Text(QStringLiteral("ERR:  Data section not active."));
		return;
	}

	count = d_laser_file.Data_Count(active_section);
	
	for(cntr = 0;cntr < count;++cntr)
	{
		data = d_laser_file.Data(active_section,cntr);
		
		if(text_lines.size() < data.size())
		{
			text_lines.resize(data.size());
		}
		
		for(iter = data.begin(),index = 0;iter != data.end();++iter,++index)
		{
			if(cntr == 0)
			{
				text = QString("%1").arg((*iter).x,0,'f',5);
				text_lines[index] = text;
			}
			
			text = QString("\t%1").arg((*iter).y,0,'f',5);
			
			text_lines[index] += text;
		}
	}
	
	text = QString();
	
	for(str_iter = text_lines.begin();str_iter != text_lines.end();++str_iter)
	{
		text += (*str_iter);
		text.push_back('\n');
	}
	
	QApplication::clipboard()->setText(text,QClipboard::Clipboard);
	
}

void TLaserDataViewer::Section_1_Toggled(
	bool								state)
{
	bool								signal_state[3];
	
	signal_state[0] = d_section_1_button->blockSignals(true);
	signal_state[1] = d_section_2_button->blockSignals(true);
	signal_state[2] = d_section_3_button->blockSignals(true);
	
	if(state)
	{
		d_section_2_button->setChecked(false);
		d_section_3_button->setChecked(false);
	}
	else if(d_section_2_button->isEnabled())
	{
		d_section_2_button->setChecked(true);
	}
	else if(d_section_3_button->isEnabled())
	{
		d_section_3_button->setChecked(true);
	}
	else
	{
		d_section_1_button->setChecked(true);
	}
	
	d_section_1_button->blockSignals(signal_state[0]);
	d_section_2_button->blockSignals(signal_state[1]);
	d_section_3_button->blockSignals(signal_state[2]);
	
	this->Update_Display();
}

void TLaserDataViewer::Section_2_Toggled(
	bool								state)
{
	bool								signal_state[3];
	
	signal_state[0] = d_section_1_button->blockSignals(true);
	signal_state[1] = d_section_2_button->blockSignals(true);
	signal_state[2] = d_section_3_button->blockSignals(true);
	
	if(state)
	{
		d_section_1_button->setChecked(false);
		d_section_3_button->setChecked(false);
	}
	else if(d_section_3_button->isEnabled())
	{
		d_section_3_button->setChecked(true);
	}
	else if(d_section_1_button->isEnabled())
	{
		d_section_1_button->setChecked(true);
	}
	else
	{
		d_section_2_button->setChecked(true);
	}
	
	d_section_1_button->blockSignals(signal_state[0]);
	d_section_2_button->blockSignals(signal_state[1]);
	d_section_3_button->blockSignals(signal_state[2]);
	
	this->Update_Display();
}

void TLaserDataViewer::Section_3_Toggled(
	bool								state)
{
	bool								signal_state[3];
	
	signal_state[0] = d_section_1_button->blockSignals(true);
	signal_state[1] = d_section_2_button->blockSignals(true);
	signal_state[2] = d_section_3_button->blockSignals(true);
	
	if(state)
	{
		d_section_1_button->setChecked(false);
		d_section_2_button->setChecked(false);
	}
	else if(d_section_1_button->isEnabled())
	{
		d_section_1_button->setChecked(true);
	}
	else if(d_section_2_button->isEnabled())
	{
		d_section_2_button->setChecked(true);
	}
	else
	{
		d_section_3_button->setChecked(true);
	}
	
	d_section_1_button->blockSignals(signal_state[0]);
	d_section_2_button->blockSignals(signal_state[1]);
	d_section_3_button->blockSignals(signal_state[2]);
	
	this->Update_Display();
}

int TLaserDataViewer::Get_Active_Section(void) const
{
	if(d_section_1_button->isChecked())
	{
		return 0;
	}
	else if(d_section_2_button->isChecked())
	{
		return 1;
	}
	if(d_section_3_button->isChecked())
	{
		return 2;
	}
	
	return -1;
}

void TLaserDataViewer::Load_Data(
	const QString						&file_name)
{
	d_laser_file.Reset();
	
	if(!d_laser_file.Import_File(file_name))
	{
		this->Clear_Display();

		this->Add_Display_Text(QString("ERR:  %1").arg(d_laser_file.Get_Last_Error()));		
		this->Enable_Options(false);
		this->Update_Section_Names();
		
		return;
	}
	else
	{
		this->Enable_Options(true);
		this->Update_Section_Names();
		this->Update_Display();
	}
}

void TLaserDataViewer::Update_Display(void)
{
	TLaserFile::TLaserDataEnvironment	environment;
	std::vector<TVector2>				data;
	std::vector<TVector2>::iterator		iter;
	double								target,deviation;
	int									data_set(0);
	QColor								color;
	int									red,green,blue;
	int									count;
	int									cntr;
	int									active_section;
	
	this->Clear_Display();

	active_section = this->Get_Active_Section();
	
	if(active_section < 0)
	{
		this->Add_Display_Text(QStringLiteral("ERR:  Data section not active."));
		return;
	}

	data = d_laser_file.Data(active_section,-1);

	// display title
	this->Add_Display_Text(QStringLiteral("Laser Data Viewer"));
	this->Add_Display_Text(QStringLiteral("======================================"));
	this->Add_Display_Text(QString());
	this->Add_Display_Text(QString("Input File Name: %1").arg(QDir::toNativeSeparators(d_laser_file.File_Name())));
	this->Add_Display_Text(QString("Input File Description: %1").arg(d_laser_file.File_Type_Description()));
	this->Add_Display_Text(QString("Active Section: %1").arg(d_laser_file.Section_Name(active_section)));
	this->Add_Display_Text(QString());
	
	if(d_laser_file.Has_Envirionmental_Data())
	{
		environment = d_laser_file.Environmental_Data();

		// display environmental data
		this->Add_Display_Text(QStringLiteral("Environmental Data"));
		this->Add_Display_Text(QStringLiteral("--------------------------------------"));
		this->Add_Display_Text(QString());
		this->Add_Display_Text(QString("Air Temperature: %1 ˚C").arg(environment.air_temperature,0,'f',2));
		this->Add_Display_Text(QString("Air Pressure   : %1 kPa").arg(environment.air_pressure,0,'f',3));
		this->Add_Display_Text(QString("Air Humidity   : %1 %").arg(environment.air_humidity,0,'f',1));
		this->Add_Display_Text(QString("Mat Temperature: %1 ˚C").arg(environment.mat_temperature,0,'f',2));
		this->Add_Display_Text(QString());
	}
	
	// display laser data
	this->Add_Display_Text(QStringLiteral("Measurement Data"));
	this->Add_Display_Text(QStringLiteral("--------------------------------------"));
	this->Add_Display_Text(QString());
	this->Add_Display_Text(QString("Number of Measurements: %1").arg(d_laser_file.Data_Count(active_section)));
	this->Add_Display_Text(QString());
	this->Add_Display_Text(QStringLiteral("    Target           Mean Deviation"));
	this->Add_Display_Text(QStringLiteral("--------------------------------------"));
	
	for(iter = data.begin();iter != data.end();++iter)
	{
		target = (*iter).x;
		deviation = (*iter).y;
		
		this->Add_Display_Text(QString("%1 %2")
							   .arg(target,11,'f',4)
							   .arg(deviation,18,'f',4));
		
	}
	
	// put top of document into view
	d_data_text->moveCursor(QTextCursor::Start);
	
	
	// update graphical view
	d_graph_display_widget->DeleteDataSets();
	d_graph_display_widget->Title(d_laser_file.File_Name_Short());
	d_graph_display_widget->HorizontalLegend(QStringLiteral("Position"));
	d_graph_display_widget->VerticalLegend(QStringLiteral("Deviation"));
	
	count = d_laser_file.Data_Count(active_section);
	
	for(cntr = 0;cntr < count;++cntr)
	{
		data = d_laser_file.Data(active_section,cntr);

		d_graph_display_widget->GenerateRGBColor(data_set,&red,&green,&blue);
		color.setRgb(red,green,blue);
		
		data_set = d_graph_display_widget->CreateDataSet(color);
		
		for(iter = data.begin();iter != data.end();++iter)
		{
			d_graph_display_widget->AddPoint(data_set,(*iter));
		}
	}
	
	d_graph_display_widget->UpdateGraph();
}

void TLaserDataViewer::Update_Section_Names(void)
{
	d_section_1_button->setText(d_laser_file.Section_Name(0));
	d_section_2_button->setText(d_laser_file.Section_Name(1));
	d_section_3_button->setText(d_laser_file.Section_Name(2));
}

void TLaserDataViewer::Clear_Display(void)
{
	d_data_text->clear();
}

void TLaserDataViewer::Add_Display_Text(
	const QString						&text)
{
	d_data_text->appendPlainText(text);
}

void TLaserDataViewer::Add_Display_Text(
	const QStringList					&text_lines)
{
	QList<QString>::const_iterator		str_iter;
	
	for(str_iter = text_lines.begin();str_iter != text_lines.end();str_iter++)
	{
		d_data_text->appendPlainText(*str_iter);
	}
}

void TLaserDataViewer::Clear_Graph(void)
{
	d_graph_display_widget->DeleteDataSets();
	d_graph_display_widget->Title(QString());
}

void TLaserDataViewer::Enable_Options(
	bool								state)
{
	d_saveas_button->setEnabled(state);
	d_copy_inverted_button->setEnabled(state);
	d_copy_normal_button->setEnabled(state);
	d_copy_all_button->setEnabled(state);
	
	d_section_1_button->setEnabled(d_laser_file.Data_Count(0) > 0);
	d_section_2_button->setEnabled(d_laser_file.Data_Count(1) > 0);
	d_section_3_button->setEnabled(d_laser_file.Data_Count(2) > 0);
	
	// make sure disabled section is not active
	// the only exception is section 1 (default)
	if(!d_section_2_button->isEnabled())
	{
		if(d_section_2_button->isChecked())
		{
			d_section_2_button->setChecked(false);
		}
	}
	
	if(!d_section_3_button->isEnabled())
	{
		if(d_section_3_button->isChecked())
		{
			d_section_3_button->setChecked(false);
		}
	}
}
